/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.backend;

import java.util.ArrayList;
import java.util.List;

/**
 * ???
 */
class InferenceNode
{
    private List temps = new ArrayList();
    private List links = new ArrayList();
    private boolean removed;
    private int color;

    InferenceNode(QTemporary temp)
    {
        temps.add(temp);
    }

    /**
     * Gets the position of this node.
     */
    public int getPosition()
    {
        return ((QTemporary)temps.get(0)).getPosition();
    }

    /**
     * Returns the number of inference.
     */
    public int countInference()
    {
        int count = 0;

        for (int i = 0; i < links.size(); i++) {
            count += ((InferenceNode)links.get(i)).isRemoved() ? 0 : 1;
        }

        return count;
    }

    /**
     * Adds a link to an other node.
     */
    public void linkTo(InferenceNode other)
    {
        if (!links.contains(other)) {
            links.add(other);
        }
    }

    /**
     * Returns the weight.
     */
    public int getWeight()
    {
        return 0;
    }
  
    /**
     * Returns the linked nodes.
     */
    public List getInferences()
    {
        return links;
    }

    /**
     * Returns the temporaries represented by this block.
     */
    public QTemporary[] getTemporaries()
    {
        return (QTemporary[])temps.toArray(new QTemporary[temps.size()]);
    }

    /**
     * Coalesce two nodes together: not implemented, throws RuntimeException.
     */
    public void coalesceTo(InferenceNode other)
    {
        throw new RuntimeException();
    }

    /**
     * Removes this node.
     */
    public void remove()
    {
        this.removed = true;
    }

    /**
     * Returns true if this node has been removed from the graph.
     */
    public boolean isRemoved()
    {
        return removed;
    }

    /**
     * Returns the size.
     */
    public int getSize()
    {
        return ((QTemporary)temps.get(0)).getSize();
    }

    /**
     * Returns the precolor.
     */
    public int getPrecolor()
    {
        return ((QTemporary)temps.get(0)).getPrecolor();
    }

    /**
     * Returns if this node is precolored.
     */
    public boolean isPrecolored()
    {
        return ((QTemporary)temps.get(0)).getPrecolor() != QTemporary.UNINITIALIZED;
    }

    /**
     * Assigns the color to temporaries.
     */
    public void setTempsColor()
    {
        for (int j = 0; j < temps.size(); j++) {
            QTemporary temp = (QTemporary)temps.get(j);

            temp.setRegister(getColor());
        }
    }
  
    /**
     * Sets the color of the node.
     */
    public void colorize()
    {
        if (isPrecolored()) {
            color = getPrecolor();
        }
        else {
            int   i = -1;
      
            if (links.size() == 0) {
                removed = false;
                color = 0;
                return;
            }
        loop:
            while (true) {
                i += 1;

                for (int j = 0; j < links.size(); j++) {
                    InferenceNode inf = (InferenceNode)links.get(j);
      
                    if (!inf.isRemoved()) { 
                        if ( inf.getColor() == i
                             || inf.getColor() + inf.getSize() -1 == i) {

                            continue loop;
                        }
                        if ( getSize() == 2 && i + 1 == inf.getColor() ) {
                            continue loop;
                        }
                    }
                }
                color = i;
                removed = false;
                return;
            }
        }
    }

    /**
     * Returns the color of the node.
     */
    public int getColor()
    {
        return color;
    }
}
