/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.backend;

import at.dms.classfile.Constants;

/**
 * This class represents the an instruction that has no return value.
 */
class QVoid
    extends QNode
{
    /**
     * The origin.
     */
    private QOrigin origin;

    /**
     * ???
     */
    public QVoid(QOrigin origin)
    {
        this.origin = origin;
    }

    /**
     * Human readable form.
     */
    public String toString()
    {
        return "V " +  origin;
    }

    /**
     * The type of this instruction.
     */
    public int getType()
    {
        return Constants.TYP_VOID;
    }

    /**
     * Returns the primitive instruction.
     */
    public InstructionHandle getInstruction()
    {
        return origin.getInstruction();
    }

    /**
     * Returns the defined temporary.
     */
    public QTemporary getDef()
    {
        return null;
    }

    /**
     * Returns the used temporaries.
     */
    public QTemporary[] getUses()
    {
        return origin.getUses();
    }

    /**
     * Returns the parameters of this instruction.
     */
    public QOrigin[] getOrigins()
    {
        return origin.getOrigins();
    }

    /**
     * ???
     */
    public void setOrigin(QOrigin origin, int i)
    {
        if (this.origin instanceof QOperator) {
            this.origin.setOrigin(origin, i);
        }
        else if (i == 0) {
            this.origin = origin;
        }
        else {
            throw new RuntimeException();
        }
    }

    /**
     * Generates instructions for this quadruple.
     *
     * @param seq
     * The code sequence of instruction.
     */
    public void generate(CodeSequence seq)
    {
        origin.generate(seq);
    }
}
