/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.compiler;

import java.util.HashMap;
import antlr.Token;
import java.util.Map;

public final class CToken extends Token
{

    /**
     * Constructs a token
     */
    public CToken(int type, String text)
    {
        super(type);
        this.text = text;
    }

    public String getText()
    {
        return text;
    }

    public void setText(String text)
    {
        this.text = text;
    }

    // --------------------------------------------------------------------
    // LOOKUP
    // --------------------------------------------------------------------

    public static CToken lookupToken(int type,
                                     char[] data,
                                     int start, int length)
    {
        CToken tok = null;

        currentToken.data = data;
        currentToken.start = start;
        currentToken.length = length;
        currentToken.type = type;
    
        tok = (CToken)table.get(currentToken);
        if (tok == null) { 
            tok = new CToken(type,
                             String.valueOf(data, start, length).intern());
            table.put(currentToken.store(), tok);
        }
 
        return tok;
    }

    static class LookupToken {
        public char[] data;
        public int    start;
        public int    length;
        public int   type;

        public LookupToken store() {
            char[] temp = new char[length];
            LookupToken store = new LookupToken();

            System.arraycopy(data, start, temp, 0, length);
            store.data = temp;
            store.length = length;
            store.type = type;

            return store;
        }

        public int hashCode() {
            int h = 0;
            int index1 = start;
            char[] val = data;

            for (int i = length - 1 ; i >= 0; i--) {
                h = (h * 37) + val[index1++];
            }

            return h * 37 + type;
        }

        public boolean equals(Object o) {
            if (!(o instanceof LookupToken)) {
                return false;
            }
            LookupToken tok = (LookupToken)o;
      
            if (length != tok.length || type != tok.type) {
                return false;
            }

            char[] data1 = this.data;
            char[] data2 = tok.data;
            int index1 = start;
            int index2 = tok.start;

            for (int i = 0; i < length; i++) {
                if (data1[index1++] != data2[index2++]) {
                    return false;
                }
            }

            return true;
        }
    }

    private static final LookupToken currentToken = new LookupToken();
    private static final Map table = new HashMap(1000);

    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

    private String  text;
}

