/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.compiler;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.PushbackReader;
import java.io.Reader;
import java.io.IOException;

/**
 * Buffer that holds a whole input file.
 */
public class InputBuffer
{
    /**
     */
    public String file;

    /**
     */
    public char[] data;

    /**
     */
    public int size;

    /**
     */
    public int pos;

    /**
     */
    public int line;

    /**
     * Constructs a new input buffer.
     *
     * @param name
     * The name of the file.
     * @param file
     * The file to read.
     * @exception IOException
     * An I/O exception occurred.
     */
    public InputBuffer(String name, File file)
        throws IOException
    {
        Reader rawReader = new FileReader(file);
        PushbackReader reader =
            new PushbackReader(new BufferedReader(rawReader));

        long fileLength = file.length();
        if (fileLength > Integer.MAX_VALUE) {
            throw new IOException("File " + file + " is too big.");
        }
        this.size = (int)fileLength;
        this.data = new char[this.size + 1];
        char[] unicode = new char[4];

        int readPos = 0;
        int tmp;
        while ((tmp = reader.read()) != -1) {
            if (tmp == '\\') {
                tmp = reader.read();
                if (tmp == 'u') {
                    // Skip extra uuuuuu after \ because they are allowed.
                    do {;} while ((tmp = reader.read()) == 'u');
                    if (tmp != -1) {
                        reader.unread((char)tmp);
                    }
                    else {
                        throw new IOException("Invalid unicode escape: no "
                                              + "more characters after \\u");
                    }

                    for (int i = 0; i < 4; i++) {
                        if ((tmp = reader.read()) == -1) {
                            throw new IOException("Invalid unicode escape: " +
                                                  " not enough characters " +
                                                  "after \\u");
                        }
                        unicode[i] = (char)tmp;
                    }

                    try {
                        this.data[readPos++] = (char)
                            Integer.parseInt(new String(unicode), 16);
                    }
                    catch (NumberFormatException e) {
                        throw new IOException("Invalid unicode escape: \\u"
                                              + new String(unicode));
                    }
                }
                else if (tmp != -1) {
                    // Leave backslash in stream if it didn't mark a unicode
                    // escape; the following character is then never a unicode
                    // escape starting character, even if it is a backslash.
                    this.data[readPos++] = '\\';
                    this.data[readPos++] = (char)tmp;
                }
                else {
                    this.data[readPos++] = '\\';
                    break;
                }
            }
            else {
                this.data[readPos++] = (char)tmp;
            }
        } 

        this.size = readPos;

        /* Ignore ^Z at end of file (see JLS 3.5) */
        if (size > 0 && this.data[size - 1] == 0x1a) {
            size--;
        }

        this.data[this.size++] = ' '; // EOT

        reader.close();

        this.file = name;
        this.pos = 0;
        this.line = 1;
    }

    /**
     * Constructs a new input buffer.
     *
     * @param file  the file to read
     * @exception IOException an I/O exception occurred
     */
    public InputBuffer(File file)
        throws IOException
    {
        this(file.getPath(), file);
    }

    /**
     * Releases cached buffer.
     */
    public void release()
    {
    }

    public final boolean isEndOfData()
    {
        return pos >= size;
    }

}
