/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.compiler;

import java.io.PrintWriter;
import java.io.Writer;

/**
 * This class implements a tabbed print writer.
 */
public class TabbedPrintWriter
{
    /**
     * Filtered print writer.
     */
    private PrintWriter p;

    /**
     * Indenting position.
     */
    protected int pos;

    /**
     * Current line numer.
     */
    protected int line;

    /**
     * Current column number.
     */
    protected int column;

    /**
     * Constructs a tabbed printer writer.
     *
     * @param fileName
     * The file into the code is generated.
     */
    public TabbedPrintWriter(Writer writer)
    {
        p = new PrintWriter(writer);

        this.pos = 0;
    }

    /**
     * Close the stream at the end.
     */
    public void close()
    {
        p.close();
    }

    public int getLine()
    {
        return line;
    }

    public int getColumn()
    {
        return column;
    }

    public int getPos()
    {
        return pos;
    }

    /**
     * Set pos.
     */
    public void setPos(int pos)
    {
        this.pos = pos;
    }

    /**
     * Increment tab.
     */
    public void add(int pos)
    {
        this.pos += pos;
    }

    /**
     * Decrement tab.
     */
    public void sub(int pos)
    {
        this.pos += pos;
    }

    /**
     * Print a new line.
     */
    public void println()
    {
        p.println();
        column = 0;
        line++;
    }

    /**
     * Print a string.
     */
    public void print(String s)
    {
        /*if (Math.max(column, pos) + s.length() > 80 && s.length() > 2) {
          println();
          }*/
        checkPos();
        p.print(s);
        column += s.length();
    }

    private void checkPos()
    {
        if (column < pos) {
            p.print(space(pos - column));
            column = Math.max(column, pos);
        }
    }

    private String space(int count)
    {
        if (count <= 0) {
            count = 1;
        }
        return spaceIn(count);
    }

    private String spaceIn(int count)
    {
        return new String(new char[count]).replace((char)0, ' ');
    }
}

