/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import java.io.File;
import java.util.List;
import at.dms.classfile.FieldInfo;

/**
 * This class represents an undefined class.
 * This class is not usable to anything, so it will sooner or later produce
 * a comprehensive error.
 */
public class CBadClass
    extends CClass
{

    /**
     * Constructs "Bad Class" instance of CClass.
     */
    public CBadClass(String qualifiedName)
    {
        super(null, null, "undefined", 0,
              getIdent(qualifiedName),
              qualifiedName,
              null, false);
    }

    /**
     * Checks if this class descends from <code>from</code>
     *
     * @param from an other CClass
     * @return true if this class inherit from "from" or equals "from"
     */
    public boolean descendsFrom(CClass from)
    {
        return false;
    }

    /**
     * Returns the named field if it exists, else <code>null</code>.
     *
     * @param ident the name of the field
     * @return the field
     */
    public final CField getField(String ident)
    {
        return null;
    }

    /**
     * Look up member classes of this class, or of superclasses or interfaces.
     *
     * @param name a TypeName (6.5.2)
     */
    public CClass lookupMemberClass(String name)
    {
        return null;
    }

    /**
     * Searches for an accessible, matching method with the provided type
     * parameters. Looks in parent hierarchy as needed.
     *
     * @param caller
     * The class of the method call expression, which is used to determine
     * whether a matching method is accessible. If it is not accessible,
     * the search goes on for another matching method. This parameter may
     * be <code>null</code> to look up any matching method, regardless of
     * accessibility.
     *
     * @param name
     * Method name.
     *
     * @param params
     * Method parameters.
     */
    public CMethod lookupMethod(CClass caller, String name, CType[] params)
    {
        return null;
    }

    /**
     * Searches for a matching method with the provided type parameters
     * in the superclass and/or interfaces of this type.
     *
     * @param caller
     * The class of the method call expression, which is used to determine
     * whether a matching method is accessible. If it is not accessible,
     * the search goes on for another matching method. This parameter may
     * be <code>null</code> to look up any matching method, regardless of
     * accessibility.
     *
     * @param name
     * Method name.
     *
     * @param params
     * Method parameters.
     */
    public CMethod lookupSuperMethod(CClass caller,
                                     String name, CType[] params)
    {
        return null;
    }

    /**
     * Searches for a field. Looks in parent hierarchy as needed.
     * @param name field name.
     */
    public CField lookupField(String name)
    {
        return null;
    }

    /**
     * Checks if this class is accessible from another class.
     *
     * @return true
     * If this class is accessible from the class <code>from</code>.
     */
    public boolean isAccessible(CClass from)
    {
        return false;
    }

    /**
     * Returns a list of interface methods.
     */
    public CMethod[] getInterfaceMethods()
    {
        return new CMethod[0];
    }

    /**
     * Returns a list of abstract methods.
     */
    public CMethod[] getAbstractMethods()
    {
        return new CMethod[0];
    }

    /**
     */
    public void collectInterfaceMethods(List l)
    {
    }

    /**
     */
    public void collectAbstractMethods(List l)
    {
    }

    /**
     * Generate the code in a class file.
     */
    public void genClassFile(File destination)
    {
        throw new RuntimeException();
    }
}
