/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import java.util.List;

import at.dms.compiler.Compiler;
import at.dms.compiler.PositionedError;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a local context during checkBody.
 * It follows the control flow and maintain informations about
 * variable (initialized, used, allocated) and exceptions (thrown, catched.)
 * It also verify that context is still reachable.
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree.
 * @see CContext
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CBodyContext
 * @see CBlockContext
 */
public class CCompilationUnitContext extends CContext
{
    /**
     * Compiler in use.
     */
    private final Compiler compiler;

    /**
     * Classes in compilation unit.
     */
    private final List classes;

    /**
     * The compilation unit.
     */
    private final CCompilationUnit cunit;

    /**
     * Construct a compilation unit context.
     */
    CCompilationUnitContext(Compiler compiler, CCompilationUnit cunit)
    {
        this(compiler, cunit, null);
    }

    /**
     * Construct a compilation unit context.
     */
    CCompilationUnitContext(Compiler compiler, 
                            CCompilationUnit cunit,
                            List classes)
    {
        this.compiler = compiler;
        this.cunit = cunit;
        this.classes = classes;
    }

    /**
     * @param field  the definition of a field
     * @return a field from a field definition in current context
     */
    public int getFieldInfo(CField field)
    {
        return 0;
    }

    /**
     * @param var  the definition of a variable
     * @return a variable info from a variable definition in current context
     */
    public int getVariableInfo(JLocalVariable variable)
    {
        return 0;
    }

    /**
     * @return a class according to imports or null if error occur
     * @exception UnpositionedError this error will be positioned soon
     */
    public CClassType lookupClass(String name) throws UnpositionedError
    {
        return cunit.lookupClass(name);
    }

    /**
     * getParentContext
     * @return the parent
     */
    public CContext getParentContext()
    {
        fail();
        return null;
    }

    /**
     * getClass
     * @return the near parent of type CClassContext
     */
    public CClassContext getClassContext()
    {
        return null;
    }

    /**
     * getMethod
     * @return the near parent of type CMethodContext
     */
    public CMethodContext getMethod()
    {
        return null;
    }

    /**
     * @return the compilation unit
     */
    public CCompilationUnitContext getCompilationUnit()
    {
        return this;
    }

    public CBlockContext getBlockContext()
    {
        return null;
    }  

    public CClassContext createClassContext(CSourceClass clazz)
    {
        return new CClassContext(this, clazz, null);
    }

    /**
     * Add an error into the list and eat it
     * This method should be called after a try catch block after catching
     * exception or directly without exception thrown.
     * @param error  the error
     */
    public void reportTrouble(PositionedError trouble)
    {
        compiler.reportTrouble(trouble);
    }

    /**
     * Gets the compiler
     */
    public Compiler getCompiler()
    {
        return compiler;
    }
  
    /**
     * Adds a class to generate
     */
    public void addSourceClass(CSourceClass clazz)
    {
        if (!classes.contains(clazz)) {
            classes.add(clazz);
        }
    }
}
