/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.PositionedError;

/**
 * This class represents a local context during checkBody
 * It follows the control flow and maintain informations about
 * variable (initialised, used, allocated), exceptions (thrown, catched)
 * It also verify that context is still reachable
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CContext
 */
public class CLabeledContext
    extends CAbstractBodyContext
{

    /**
     * Construct a block context, it supports local variable allocation
     * throw statement and return statement
     * @param parent  the parent context, it must be different
     *    than null except if called by the top level
     */
    CLabeledContext(CBodyContext parent, JLabeledStatement self)
    {
        super(parent);

        this.self = self;
    }


    /**
     * Adds a break from within this context to the specified statement.
     */
    protected void addForwardedBreak(JStatement breakTo, CBodyContext from)
        throws PositionedError
    {
        if (breakTo == self) {
            self.addBreak(from);
        }
        else {
            super.addForwardedBreak(breakTo, from);
        }
    }

    /**
     * Adds a continue from within this context to the specified statement.
     */
    protected void addForwardedContinue(JStatement continueTo,
                                        CBodyContext from)
        throws PositionedError
    {
        if (continueTo == self) {
            self.addContinue(from);
        }
        else {
            super.addForwardedContinue(continueTo, from);
        }
    }

    /**
     * Creates a body context with this context as its parent.
     */
    public CBodyContext createBodyContext()
    {
        return new CBodyContext(this);
    }

    /**
     * Returns the statement with the specified label.
     */
    public JLabeledStatement getLabeledStatement(String label)
    {
        if (label.equals(self.getLabel())) {
            return self;
        }
        else {
            return ((CBodyContext)parent).getLabeledStatement(label);
        }
    }


    private final JLabeledStatement self;
}
