/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.classfile.MethodInfo;
import at.dms.classfile.ConstantPool;
import at.dms.classfile.ClassFileFormatException;
import at.dms.classfile.CodeInfo;

/**
 * VMS 4.6: Methods
 *
 * Each method, and each (class or instance) initialization method is
 * described by this structure.
 */
public class CMethodInfo extends MethodInfo
{

    /**
     * Constructs a method entry
     *
     * @param modifiers access permission to and properties of the method
     * @param name  the simple name of the method (or <init> or <clinit>)
     * @param type  the method signature
     * @param exceptions the checked exceptions the method may throw
     * @param code  the virtual machine instructions and auxiliary infos
     * @param deprecated is this method deprecated ?
     * @param synthetic is this method synthesized by the compiler ?
     *
     * NOTE:
     * VMS 4.7.5: There must be exactly one Exceptions attribute in each
     * method_info structure.
     */
    public CMethodInfo(short modifiers,
                       String name,
                       String type,
                       String[] exceptions,
                       CSourceMethod method,
                       boolean deprecated,
                       boolean synthetic)
    {
        super(modifiers, name, type, exceptions,
              CodeInfo.DUM_INFO, deprecated, synthetic);
        this.method = method;
    }

    /**
     * Insert or check location of constant value on constant pool
     *
     * @param cp  the constant pool for this class
     */
    public void resolveConstants(ConstantPool cp)
        throws ClassFileFormatException
    {
        CodeInfo code = method.genCode();
        setCodeInfo(code);
        super.resolveConstants(cp);
    }

    private CSourceMethod method;
}
