/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

/**
 * This class represents a special outer-this class type in the type structure.
 * This is a reference to a class type that is not known yet because the
 * type of the outer class is not known at this point. As soon as it is
 * known, this special class starts to act like a normal CClassNameType.
 */
public class COuterThisType
    extends CClassNameType
{
    /**
     * The declaration of the type of the outer-this.
     */
    private JTypeDeclaration  decl;

    /**
     * Construct a outer-this class type.
     *
     * @param decl the type of the outer-this.
     */
    public COuterThisType(JTypeDeclaration decl)
    {
        super("not-yet-defined");

        this.decl = decl;
    }

    /**
     * Check that type is valid.
     * Necessary to resolve String into java/lang/String
     *
     * @param context
     * The context (may be be null.)
     */
    public void checkType(CContext context)
    {
        if (decl != null) {
            setQualifiedName( decl.getCClass().getOwner()
                              .getType().getQualifiedName() );
            decl = null;
        }
    }
}
