/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.compiler.FastStringBuffer;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents java and kopi numericals types
 * Such as byte, short, int, long, float, double
 */
final class CVoidType extends CType
{

    /**
     * Constructor
     */
    protected CVoidType()
    {
        super(TID_VOID);
    }

    /**
     * check that type is valid
     * necessary to resolve String into java/lang/String
     * @exception UnpositionedError this error will be positioned soon
     */
    public void checkType(CContext context) throws UnpositionedError {}

    /**
     * isAssignableTo
     * @param dest  the destination type
     * @return  assignment is valid ?
     */
    public boolean isAssignableTo(CType dest)
    {
        return false;
    }

    /**
     * isCastableTo
     * @param dest  the destination type
     * @return  cast is valid ?
     */
    public boolean isCastableTo(CType dest)
    {
        return false;
    }

    /**
     * Transforms this type to a string
     */
    public String toString()
    {
        return "void";
    }

    /**
     * Transforms this type to a string
     */
    public String getSignature()
    {
        return "V";
    }

    /**
     * Transforms this type to a string
     */
    protected void appendSignature(FastStringBuffer buff)
    {
        buff.append('V');
    }

    /**
     * Returns the size used in stack by value of this type
     */
    public int getSize()
    {
        return 0;
    }
}
