/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * JLS 14.7 ?? : Expression List Statement (not: Expression Statement)
 *
 * Note: the following applies to expression statements; I'm not sure if
 * it applies to expression list statements too - EJB.
 *
 * Certain kinds of expressions may be used as statements by following them
 * with semicolon.
 * An expression statement is executed by evaluating the expression; if the
 * expression has a value, the value is discarded.
 */
public class JExpressionListStatement
    extends JStatement
{

    /**
     * Construct a node in the parsing tree
     * @param where  the line of this node in the source code
     * @param expr  the expression to evaluate.
     */
    public JExpressionListStatement(TokenReference where,
                                    JExpression[] exprs,
                                    JavaStyleComment[] comments)
    {
        super(where, comments);
        this.exprs = exprs;
    }

    /**
     * Returns an array of expression
     */
    public JExpression[] getExpressions()
    {
        return exprs;
    }

    /**
     * Check statement.
     *
     * @param context the actual context of analyse
     * @exception PositionedError if the check fails
     * @exception UnpositionedError this error will be positioned soon
     */
    public void checkStatement(CBodyContext context)
        throws PositionedError
    {
        for (int i = 0; i < exprs.length; i++) {
            //!!! ADD PARAMETERS !!! graf 991208 XXX
            CExpressionContext expression = context.createExpressionContext();

            expression.setDiscardValue(true);
            exprs[i] = exprs[i].checkExpression(expression);
            expression.setDiscardValue(false); // not necessary
        }
    }

    /**
     * Accepts the specified visitor
     * @param p the visitor
     */
    public void accept(KjcVisitor p)
    {
        super.accept(p);
        p.visitExpressionListStatement(this, exprs);
    }

    /**
     * Generates a sequence of bytecodes
     * @param code the code list
     */
    public void genCode(CodeSequence code)
    {
        code.setLineNumber(getTokenReference().getLine());

        for (int i = 0; i < exprs.length; i++) {
            code.setDiscardValue(true);
            exprs[i].genCode(code);
            code.setDiscardValue(false);
        }
    }

    private JExpression[] exprs;
}
