/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a new allocation expression 'new toto(1)'
 */
public class JNewArrayExpression
    extends JExpression
{

    /**
     * Construct a node in the parsing tree
     * This method is directly called by the parser
     * @param where  the line of this node in the source code
     * @param baseType  the type of this array allocator
     * @param dims  dimensions and initializers
     */
    public JNewArrayExpression(TokenReference where,
                               CType baseType,
                               JArrayDimsAndInits dims)
    {
        super(where);

        this.baseType = baseType;
        this.type = baseType;
        this.dims = dims;
    }

    /**
     * Compute the type of this expression (called after parsing)
     * @return the type of this expression
     */
    public CType getType()
    {
        return type;
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context the actual context of analyse.
     * @return a pure java expression including promote node.
     * @exception PositionedError if the check fails.
     */
    public JExpression checkExpression(CExpressionContext context)
        throws PositionedError
    {
        try {
            baseType.checkType(context);
        }
        catch (UnpositionedError e) {
            throw e.addPosition(getTokenReference());
        }

        type = dims.checkExpression(context, baseType);

        try {
            type.checkType(context);
        }
        catch (UnpositionedError e) {
            throw e.addPosition(getTokenReference());
        }

        return this;
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitNewArrayExpression(this, baseType, dims);
    }

    /**
     * Generates a sequence of bytecodes
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        code.setLineNumber(getTokenReference().getLine());

        dims.genCode(code);
    }

    protected CType baseType;
    protected CType type;
    protected JArrayDimsAndInits dims;
}
