/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * A 'super' expression.
 */
public class JSuperExpression
    extends JThisExpression
{
    /**
     * Construct a node in the parsing tree.
     *
     * @param where the line of this node in the source code.
     */
    public JSuperExpression(TokenReference where)
    {
        super(where);
    }

    /**
     * Construct a node in the parsing tree.
     *
     * @param where
     * The line of this node in the source code.
     * @param prefix
     * The class in a "ClassName.super" construct.
     */
    public JSuperExpression(TokenReference where, JExpression prefix)
    {
        super(where, prefix);
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context the actual context of analyse
     * @return  a pure java expression including promote node
     * @exception PositionedError  if the check fails
     * @exception UnpositionedError this error will be positioned soon
     */
    public JExpression checkExpression(final CExpressionContext context)
        throws PositionedError
    {
        if (getPrefix() != null) {
            setPrefix(super.checkExpression(context));
        }
        else if (getSelf() == null) {
            setSelf(context.getClassContext().getCClass().getSuperClass());
        }
        return this;
    }

    /**
     * Accepts the specified visitor.
     *
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitSuperExpression(this);
    }

    /**
     * Generates a sequence of bytecodes.
     *
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        if (getPrefix() != null) {
            getPrefix().genCode(code);
        }
        else {
            code.plantLoadThis();
        }
    }
}

