/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * JLS 14.3: Local type declaration declaration statement
 *
 * A local type declaration declaration statement declares one type
 * declaration in a body of a method.
 */
public class JTypeDeclarationStatement
    extends JStatement
{

    /**
     * Construct a node in the parsing tree.
     *
     * @param where
     * The line of this node in the source code.
     * @param decl
     * The type declaration in this statement.
     */
    public JTypeDeclarationStatement(TokenReference where,
                                     JTypeDeclaration decl)
    {
        super(where, null);
        this.decl = decl;
    }

    /**
     * Check statement.
     *
     * @param context
     * The actual context of analyse.
     * @exception PositionedError
     * If the check fails.
     */
    public void checkStatement(CBodyContext context)
        throws PositionedError
    {
        CClass owner = context.getClassContext().getCClass();
        String prefix =
            owner.getQualifiedName() + "$"
            + context.getClassContext().getNextSyntheticIndex() + "$";

        decl.generateInterface((Main)context.getCompiler(), owner, prefix);

        try {
            context.getBlockContext().addClass(decl.getCClass());
        }
        catch (UnpositionedError cue) {
            throw cue.addPosition(getTokenReference());
        }

        decl.checkInterface(context);
        decl.checkInitializers(context);
        decl.checkTypeBody(context);
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p) {
        super.accept(p);
        p.visitTypeDeclarationStatement(this, decl);
    }

    /**
     * Generates a sequence of bytecodes
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        // nothing to do here
    }

    private JTypeDeclaration decl;
}
