/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.classfile.ClassRefInstruction;
import at.dms.compiler.PositionedError;

/**
 * This class convert arithmetics expression from types to types.
 * This is always a generated node in the syntax tree, it does not result
 * directly from parsing.
 */
public class JUnaryPromote
    extends JExpression
{
    /**
     * The expression to promote.
     */
    private JExpression expr;

    /**
     * The type to promote to.
     */
    private CType type;

    /**
     * Construct a node in the parsing tree.
     *
     * @param expr
     * The expression to promote.
     * @param type
     * The type to promote to.
     */
    public JUnaryPromote(JExpression expr, CType type)
    {
        super(expr.getTokenReference());

        this.expr = expr;
        this.type = type;
        assert(type != null);
    }

    /**
     * Compute the type of this expression. (called after parsing)
     *
     * @return
     * The type of this expression.
     */
    public CType getType()
    {
        return type;
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context
     * The actual context of analyse.
     *
     * @return
     * A pure java expression including promote node.
     *
     * @exception PositionedError
     * If the check fails.
     */
    public JExpression checkExpression(CExpressionContext context)
        throws PositionedError
    {
        if (type.equals(CStdType.String)
            && expr.getType().isReference()
            && !(expr instanceof JNullLiteral)) {

            JNameExpression name = new JNameExpression
                (getTokenReference(),
                 new JCheckedExpression(getTokenReference(), expr),
                 "toString");

            return new JMethodCallExpression(getTokenReference(),
                                             name, JExpression.EMPTY)
                .checkExpression(context);
        }

        return this;
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        if (!expr.isAssignableTo(type)) {
            p.visitUnaryPromoteExpression(this, expr, getType());
        }
        else {
            expr.accept(p);
        }
    }

    /**
     * Generates a sequence of bytecodes
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        expr.genCode(code);

        if (type.isNumeric()) {
            ((CNumericType)expr.getType()).genCastTo((CNumericType)type, code);
        }
        else if (!expr.isAssignableTo(type)) {
            code.plantInstruction
                (new ClassRefInstruction(opc_checkcast,
                                         ((CClassType)type)
                                         .getQualifiedName()));
        }
    }
}
