/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.optimize;

import at.dms.classfile.AccessorContainer;
import at.dms.classfile.AccessorTransformer;
import at.dms.classfile.BadAccessorException;
import at.dms.classfile.Instruction;
import at.dms.classfile.InstructionAccessor;
import at.dms.util.ArrayLocator;

/**
 * This class replaces all references to instructions by their associated
 * handle.
 * At the same time, it adds line numbers to the handles and checks if they
 * are references by an instruction, handler or local variable info.
 */
class HandleCreator
    implements AccessorTransformer
{
    /**
     * ...
     */
    private final ArrayLocator locator;

    /**
     * ...
     */
    private final InstructionHandle[] handles;

    /**
     * Creates a new HandleCreator object.
     *
     * @param instructions
     * The array of instructions.
     *
     * @param handles
     * The array of instruction handles.
     */
    public HandleCreator(Instruction[] instructions,
                         InstructionHandle[] handles)
    {
        this.locator = new ArrayLocator(instructions);
        this.handles = handles;
    }

    /**
     * Transforms the specified accessor.
     *
     * @param accessor
     * The accessor to transform.
     *
     * @return
     * The transformed accessor.
     */
    public InstructionAccessor transform(InstructionAccessor accessor,
                                         AccessorContainer container)
        throws BadAccessorException
    {
        int index = locator.getIndex(accessor);

        if (index == -1) {
            throw new BadAccessorException("not in array: container: "
                                           + container + ", accessor: "
                                           + accessor);
        }

        handles[index].attachTo(container);    

        return handles[index];
    }
}
