/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.ksm;

import java.text.MessageFormat;

/**
 * This class is the entry point for the KOPI assembler.
 */
public class Main
{
    /**
     * Command-line options.
     */
    protected KsmOptions options;

    /**
     * Only main can construct Main.
     */
    private Main(String[] args)
    {
        if (!parseArguments(args)) {
            System.exit(1);
        }

        String[] infiles = options.nonOptions;

        if (infiles.length == 0) {
            options.usage();
            System.err.println(KsmMessages.NO_SOURCE_FILE);
            System.exit(1);
        }
        else {
            boolean errorsFound = false;

            for (int i = 0; i < infiles.length; i++) {
                if (options.verbose) {
                    System.err.println(MessageFormat.format
                                       (KsmMessages.PROCESSING.getFormat(),
                                        new String[] { infiles[i] }));
                }

                try {
                    Assembler asm = new Assembler();

                    asm.assemble(infiles[i], options.destination,
                                 options.optimize);
                }
                catch (KsmError e) {
                    System.err.println(e.getMessage());
                    errorsFound = true;
                }
            }

            System.exit(errorsFound ? 1 : 0);
        }
    }

    /**
     * Entry point to the assembler.
     */
    public static void main(String[] args)
    {
        new Main(args);
    }

    /**
     * Parse command line arguments.
     */
    private boolean parseArguments(String[] args)
    {
        options = new KsmOptions();
        return options.parseCommandLine(args);
    }
}
