/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.optgen;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;

/**
 * This class is the entry point for the Options java-source generator.
 */
public class Main
{
    /**
     * Suffix for the output file names.
     */
    private final static String OUTPUTFILE_SUFFIX = "Options.java";

    /**
     * Names of source files.
     */
    private String[] sourceFiles;
    
    /**
     * Parsed source files.
     */
    private DefinitionFile[] definitions;

    /**
     * Only main can construct Main.
     */
    private Main(String[] args)
    {
        parseArgs(args);
        definitions = new DefinitionFile[sourceFiles.length];

        parseSourceFiles();
        checkIdentifiers();
        checkShortcuts();
        buildInterfaceFile();

        System.exit(0);
    }

    /**
     *
     */
    private void parseSourceFiles()
    {
        boolean errorsFound = false;
        
        for (int i = 0; i < sourceFiles.length; i++) {
            try {
                definitions[i] = DefinitionFile.read(sourceFiles[i]);
            }
            catch (OptgenError e) {
                System.err.println(e.getMessage());
                errorsFound = true;
            }
        }
            
        if (errorsFound) {
            System.exit(1);
        }
    }


    /**
     *
     */
    private void checkIdentifiers()
    {
        boolean  errorsFound = false;

        for (int i = 0; i < sourceFiles.length; i++) {
            try {
                definitions[i].checkIdentifiers(new HashMap());
            }
            catch (OptgenError e) {
                System.err.println(e.getMessage());
                errorsFound = true;
            }
        }
            
        if (errorsFound) {
            System.exit(1);
        }
    }

    /**
     *
     */
    private void checkShortcuts()
    {
        boolean     errorsFound = false;

        for (int i = 0; i < sourceFiles.length; i++) {
            try {
                definitions[i].checkShortcuts(new HashMap());
            }
            catch (OptgenError e) {
                System.err.println(e.getMessage());
                errorsFound = true;
            }
        }

        if (errorsFound) {
            System.exit(1);
        }
    }

    /**
     *
     */
    private void buildInterfaceFile()
    {
        for (int i = 0; i < sourceFiles.length; i++) {
            String prefix = definitions[i].getPrefix();
            File outputFile = new File(prefix + OUTPUTFILE_SUFFIX);

            try {
                PrintWriter out = new PrintWriter(new FileWriter(outputFile));

                definitions[i].printFile(out);

                out.flush();
                out.close();
            }
            catch (IOException e) {
                System.err.println("I/O Exception on "
                                   + outputFile.getPath() + ": "
                                   + e.getMessage());
                System.exit(1);
            }
        }
    }

    /**
     * Entry point to the assembler
     */
    public static void main(String[] args)
    {
        new Main(args);
    }

    /**
     * Parse command line arguments.
     */
    private void parseArgs(String[] args)
    {
        this.sourceFiles = new String[args.length];
        System.arraycopy(args, 0, this.sourceFiles,
                         0, this.sourceFiles.length);
    }
}
